<?php
/**
* @file
* Defines a Test Case for Drupal Hook Tests. 
*
* @package DrupalTest
* @author Christopher Hopper <christopher.jf.hopper@gmail.com>
*/

/**
* DrupalTest TestCase
* 
* Defines the fixtures that are required or assist when running 
* multiple Drupal tests using PHPUnit. 
*/
class DrupalTest_Hook_TestCase extends DrupalTest_TestCase {
  /**
  * The module whose hook implementations you want to test. 
  * 
  * @var string
  */
  public $module_name = '';
  
  /**
  * The output of Drupal hook_menu, both cached and not cached. 
  * The menu items are in a multi-dimensional array: 
  * 
  * - $this->hook_menu['cached'] the cached menu items
  * - $this->hook_menu['not cached'] the menu items that are not cached 
  * 
  * @var array
  */
  public $hook_menu = array();
  
  /**
  * The output of Drupal hook_forms. 
  * 
  * @var array
  */
  public $hook_forms = array();
  
  /**
  * A list of all the module's defined constants. 
  * 
  * @var array
  */
  public $module_constants = array();
  
  /**
  * Find the forms that use a particular callback function. 
  * 
  * Takes an array of form definitions as returned by hook_forms and 
  * returns the form_ids of those that use a nominated callback 
  * function.
  * 
  * @param array $hook_forms
  *   An array of forms like those returned by hook_forms. 
  * @param string $callback
  *   The name of a callback function that you're searching for. 
  * @return array
  *   An array of form_ids
  * @todo Move this into the parent DrupalTest_TestCase class where it 
  *   can be shared and with all tests. 
  */
  public static function get_child_forms($hook_forms, $callback) {
    $forms = array();
    foreach ($hook_forms as $form_id => $val) {
      if ($val['callback'] == $callback) $forms[] = $form_id;
    }

    return $forms;
  }
  
  /**
  * Set up environment for a unit test case
  *
  * @return void
  */
  protected function setUp() {
    // Assert that the module name has been set for this instance of 
    // our class before proceeding.
    $this->assertFalse(
      empty($this->module_name), 
      'The "module name" property, $this->module_name, must be set when using the test case class for hook functions.'
    );

    parent::setUp();
    
    // Populate our fixtures with the output of our module hook 
    // functions. 
    $module = $this->module_name;
    if ($module) {
      if (function_exists($module .'_menu')) {
        // Populate the Drupal menu items fixture for this module. 
        $this->hook_menu['not cached'] = call_user_func($module .'_menu', FALSE);
        $this->hook_menu['cached'] = call_user_func($module .'_menu', TRUE);
      }
      else {
        // Populate the Drupal menu items fixture with an empty array. 
        $this->hook_menu['not cached'] = $this->hook_menu['cached'] = array();
      }
      
      if (function_exists($module .'_forms')) {
        // Populate the Drupal forms fixture for this module. 
        $this->hook_forms = call_user_func($module .'_forms');
      }
      else {
        // Populate the Drupal forms fixture with an empty array. 
        $this->hook_forms = array();
      }
    }
  }

  /**
  * Tear down environment for a unit test case
  *
  * @return void
  */
  protected function tearDown() {
    parent::tearDown();
  }
  
  /**
  * Test the implementation of hook_menu. 
  * @group hooks
  */
  public function test_hook_menu() {
    // Make assertions about the cached menu items. 
    $menu_constants = $this->get_module_constants('menu');
    $menu_items = $this->hook_menu['cached'];
    foreach ($menu_items as $item) {
      if (isset($item['callback'])) {
        // Make assertions about the callback
        $function_name = $item['callback'];
        $this->assertTrue(
          function_exists($function_name), 
          'The \'callback\' must be a function that exists.'
        );
      }
      if (isset($item['type'])) {
        // Make assertions about the menu type. 
        $menu_type = $item['type'];
        // The menu type must match one of the menu constants. 
        $type_match = FALSE; 
        foreach ($menu_constants as $type) {
          if ($menu_type == $type) {
            $type_match = TRUE;
            break;
          }
        }
        $this->assertTrue(
          $type_match, 
          'The menu \'type\' must match one of the defined constants for Menu module types.'
        );
      }
    }
  }
  
  /**
  * Test the implementation of hook_forms. 
  * @group hooks
  */
  public function test_hook_forms() {
    foreach ($this->hook_forms as $form_name => $form_properties) {
      // Test the callback specified for each form. 
      $this->assertTrue(
        function_exists($form_properties['callback']),
        "The form '$form_name' points to a callback function '${form_properties['callback']}' that does not exist."
      );
    }
  }

}

