<?php
/**
* @file
* Defines a Test Case for all Drupal Tests. 
*
* @package DrupalTest
* @author Christopher Hopper <christopher.jf.hopper@gmail.com>
*/

/**
* DrupalTest TestCase
* 
* Defines the fixtures that are required or assist when running 
* multiple Drupal tests using PHPUnit. 
*/
class DrupalTest_TestCase extends PHPUnit_Framework_TestCase {
  /**
  * A location where we can store the working directory we started in 
  * before we put ourselves in the root path of the drupal install. 
  * 
  * @var string
  */
  protected $currentWorkingDir;
  
  /**
  * Disable backup and restore operations for the global and super-
  * global variables. This prevents breaking references to MySQL-Link 
  * resources such as in the Drupal variable $GLOBALS['active_db']
  * 
  * @link http://sebastian-bergmann.de/archives/797-Global-Variables-and-PHPUnit.html 
  *   Global Variables and PHPUnit - Sebastian Bergmann 
  * @var boolean
  */
  protected $backupGlobals = FALSE;
  
  /**
  * Constant name filter. 
  * 
  * Set this variable before using the self::_constants_filter_callback 
  * method to filter an array of constants. 
  * 
  * @var string
  */
  private $_constant_name_filter = '';
  
  /**
   * Set up environment for a unit test case
   *
   * @return void
   */
  protected function setUp() {
    parent::setUp();

    // When using the command line test runner, put ourselves in the 
    // correct working directory for Drupal. 
    if (PHP_SAPI == 'cli') {
      $working_dir = getcwd();
      $drupal_root = DrupalTest_Bootstrap::getInstance()->drupalRoot;
      if ($working_dir != $drupal_root) {
        // Save the current working directory. 
        $this->currentWorkingDir = $working_dir;
        // Put us in the root path of the drupal install. 
        chdir($drupal_root);
      }
    }

    /** 
     * Supress devel footer output.
     * 
     * @global boolean devel_shutdown
     * @see devel_shutdown
     */
    $GLOBALS['devel_shutdown'] = FALSE;
  }

  /**
   * Tear down environment for a unit test case
   *
   * @return void
   */
  protected function tearDown() {
    parent::tearDown();

    // When using the command line test runner, we find a working 
    // directory has been saved, return us to that working directory 
    // that we started in. 
    if (PHP_SAPI == 'cli' && !empty($this->currentWorkingDir)) {
      chdir($this->currentWorkingDir);
    }
  }

  /**
  * Array Filter callback for Channel Nine module to help filter an 
  * array of constants.
  * 
  * @access private
  * 
  * @param string $var
  *   An array item passed in by the array_filter function. 
  * @return boolean
  *   This is an array_filter callback, so returns TRUE if the item is 
  *   to be kept in the array or FALSE if the item is not to be kept. 
  * @see array_filter 
  *   self::get_module_constants 
  */
  private function _constants_filter_callback($var) {
    if (is_string($this->_constant_name_filter) 
        && !empty($this->_constant_name_filter) 
        && strpos($var, $this->_constant_name_filter) !== FALSE) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }
  
  /**
  * Get the constants that are defined for a module. 
  * 
  * @param string $module_name
  *   The name of the module whose constants you want to list. 
  * @return array
  */
  public function get_module_constants($module_name) {
    // Define the constant name prefix we're dealing with. 
    $module_name = strtoupper($module_name) .'_';
    // Get all the constants
    $defined_constants = get_defined_constants();
    $defined_constant_names = array_keys($defined_constants);
    $this->_constant_name_filter = $module_name;
    $constant_names = array_filter($defined_constant_names, array($this, '_constants_filter_callback'));
    $constants = array_intersect_key($defined_constants, array_flip($constant_names));
    return $constants;
  }
  
}